/* File FFEEDFW.C Procedures for Feedforward networks */

#ifndef FFEEDFW_C
#define FFEEDFW_C

#include <conio.h>
#include <math.h>
#include <alloc.h>

#include "ut.h"
#include "ff.h"

#define DEBUG
void allocate_network (ff_setup_rec setup,network_rec *network);
void free_network (network_rec *network);
float ff_error (f_vector desired,f_vector net_out,ff_setup_rec setup);
void feedforward (network_rec the_network,weights_rec the_weights,
     ff_setup_rec the_setup_rec);

/* Note that no_of_inputs does not include the threshold node */
/* Note:  All inputs, outputs, hidden nodes, and weights  */
/*        have starting index = 1 */

/* Implementation */

void allocate_network (ff_setup_rec setup,network_rec *network) {
    /*  Allow 1 extra position for threshold node  */
     network->x_inp = allocate_f_vector (1,setup.no_of_inputs + 1);
     network->y_out = allocate_f_vector (1,setup.no_of_outputs);
     network->h1 = allocate_d_vector (1,setup.no_of_h1_nodes);
     network->allocated = TRUE;
     return;
     }

void free_network (network_rec *network) {
     if (network->allocated) {
	free_f_vector (network->x_inp,1);
	free_f_vector (network->y_out,1);
	free_d_vector (network->h1,1);
	}
     network->allocated = FALSE;
     return;
     }

double sigmoid(double x) {
    if (x > 50.0) return 1.0;
    if (x < -50.0) return 0.0;
    return  1.0/(1.0 + exp (-x));
    }  /* end function */

float ff_error (f_vector desired,f_vector net_out,ff_setup_rec setup) {
    int i;
    float sum = 0.0;
    for (i = 1;i <= setup.no_of_outputs;i++)
	  sum += (desired[i] - net_out[i]) *
	     (desired[i] - net_out[i]);
    return 0.5 * sum;
    }  /* end function */

void feedforward (network_rec net,weights_rec weights,
    ff_setup_rec setup) {
    int i,j;
    double wx = 0.0,wh1 = 0.0;
#ifdef DEBUG
    /*  These checks can be removed for better performance when */
    /*  you're sure everything is working OK.  */
    if (!net.allocated) {
       cprintf ("\r\nFeedforward: Network not allocated.");
       getch();
       return;
       }
    if (!weights.allocated) {
       cprintf ("\r\nFeedforward: Weights not allocated.");
       getch();
       return;
       }
#endif
    net.x_inp[setup.no_of_inputs + 1] = 1.0;  /* threshold node */
    for (i=1;i<=setup.no_of_h1_nodes;i++) {
	wx = 0.0;
	for (j=1;j<=(setup.no_of_inputs + 1);j++)
	    wx += weights.wi[j][i]*net.x_inp[j];
	net.h1[i] = sigmoid(wx);
	}  /* i */
    for (i=1;i<=setup.no_of_outputs;i++) {
	wh1 = 0.0;
	for (j=1;j<=setup.no_of_h1_nodes;j++)
	       wh1 += weights.wo[j][i]*net.h1[j];
	if (setup.sigmoid_out)
	      net.y_out[i] = sigmoid(wh1);
	   else net.y_out[i] = wh1;
	   } /* i */
    return;
    } /* end proc */

#endif