/*  File FFBRMCTR.C  Create traininf file for broom balancing system  */
/*                   Only max-min data are needed, since on-line      */
/*                   training is used.                                */

#ifndef FFBRMCTR_C
#define FFBRMCTR_C

#include <conio.h>
#include <math.h>
#include <alloc.h>
#include "ut.h"
#include "ffbrm.h"

/* Implementation */

void create_training_file (ff_setup_rec setup) {
     unsigned short i,j,k;
     int no_of_training_items = 100;
     FILE *outfile;
     float force;
     state_rec state,new_state,min_state,max_state;
     f_vector max_in,min_in,max_out,min_out;

     /* Main Procedure */
     clrscr();
     textcolor (YELLOW);
     gotoxy (1,1);
     cputs ("Max-min Data for Broom Balancing Neural Net System");
     gotoxy (1,3);
     if (setup.files.training_file_name[0] == '\0') {
	cprintf ("\r\nOutput Training file name is blank.\r\n");
	cputs ("Press key to cancel: ");
	getch();
	return;
	}
     cprintf ("Output training file name: %s",setup.files.training_file_name);
     gotoxy (1,5);
     cprintf ("Network Nodes (Input, Output): %3d,  %2d",setup.no_of_inputs,
       setup.no_of_outputs);
     gotoxy (1,7);
     cputs ("Press any key to start (esc to cancel): ");
     if (getch() == ESC_KEY) return;
     if (!open_output_text_file (&outfile,setup.files.training_file_name))
	return;
     clrscr();
     min_state.x_pos = x_pos_min;
     max_state.x_pos = x_pos_max;
     min_state.theta = theta_low_lim;
     max_state.theta = theta_up_lim;
     min_state.x_dot = 0.0;
     max_state.x_dot = 0.0;
     min_state.theta_dot = 0.0;
     max_state.theta_dot = 0.0;
     set_broom_at_start_position(&state);
     for (i = 1;i<=no_of_training_items;i++) {
	 force = force_mag * (2.0 * frand() - 1.0);
	 new_broom_state (force,state,&new_state);
	 /* x_pos max and min and theta max and min are fixed  */
	if (new_state.x_dot < min_state.x_dot)
	   min_state.x_dot = new_state.x_dot;
	if (new_state.x_dot > max_state.x_dot)
	   max_state.x_dot = new_state.x_dot;
	if (new_state.theta_dot < min_state.theta_dot)
	   min_state.theta_dot = new_state.theta_dot;
	if (new_state.theta_dot > max_state.theta_dot)
	   max_state.theta_dot = new_state.theta_dot;
	if ((new_state.x_pos > x_pos_min) &&
	    (new_state.x_pos < x_pos_max) &&
	    (new_state.theta > theta_low_lim) &&
	    (new_state.theta < theta_up_lim))
	   state = new_state;
	else set_broom_at_start_position (&state);
	}  /* i */
    /* Force max-min symmetry:  */
    if (fabs (min_state.x_dot) > max_state.x_dot)
       max_state.x_dot = fabs (min_state.x_dot);
    min_state.x_dot = -max_state.x_dot;
    if (fabs (min_state.theta_dot) > max_state.theta_dot)
       max_state.theta_dot = fabs (min_state.theta_dot);
    min_state.theta_dot = -max_state.theta_dot;

    min_in = allocate_f_vector (1,setup.no_of_inputs);
    max_in = allocate_f_vector (1,setup.no_of_inputs);
    min_out = allocate_f_vector (1,setup.no_of_outputs);
    max_out = allocate_f_vector (1,setup.no_of_outputs);

    max_in [1] = max_state.x_pos;
    min_in [1] = min_state.x_pos;
    max_in [2] = max_state.x_dot;
    min_in [2] = min_state.x_dot;
    max_in [3] = max_state.theta;
    min_in [3] = min_state.theta;
    max_in [4] = max_state.theta_dot;
    min_in [4] = min_state.theta_dot;
    max_in [5] = force_mag;
    min_in [5] = -force_mag;
    max_out [1] = max_state.theta;
    min_out [1] = min_state.theta;
    fprintf (outfile,"%3d  %2d\n",setup.no_of_inputs,setup.no_of_outputs);
    fprintf (outfile,"minimum\n");
    for (i = 1;i<=setup.no_of_inputs;i++)
	fprintf (outfile,"%11.8f\n",min_in [i]);
    for (i = 1;i<=setup.no_of_outputs;i++)
	fprintf (outfile,"%11.8f\n",min_out [i]);
    fprintf (outfile,"maximum\n");
    for (i = 1;i<=setup.no_of_inputs;i++)
	fprintf (outfile,"%11.8f\n",max_in [i]);
    for (i = 1;i<=setup.no_of_outputs;i++)
	fprintf (outfile,"%11.8f\n",max_out[i]);
    fprintf (outfile,"training data\n");
   /* Training will be done 'on-line', so we don't put any data in this file */
   fclose (outfile);
   free_f_vector (max_out,1);
   free_f_vector (min_out,1);
   free_f_vector (max_in,1);
   free_f_vector (min_in,1);
   gotoxy (1,16);
   cputs  ("Done.");
   gotoxy (1,18);
   cputs ("Press key to clear: ");
   getch();
   textmode (LASTMODE);
   return;
   }  /* end proc */

#endif