/*  File FFPRUN.C  Run a feedforward network for pattern recognition  */

#ifndef FFPRUN_C
#define FFPRUN_C

#include <conio.h>
#include <alloc.h>

#include "ut.h"
#include "ffpr.h"

void run_pattern_network(ff_setup_rec setup,pattern_display_data *disp_data);

/*  Implementation  */

char get_command (void) {
   const
      comx = 30,
      comy = 19;
   char command = ' ';
      gotoxy (comx,comy);
      cputs ("e: Edit Pattern");
      gotoxy (comx,comy + 1);
      cputs ("r: Run Network");
      gotoxy (comx,comy + 2);
      cputs ("n: Get New Pattern");
      gotoxy (comx,comy + 3);
      cputs ("x: Exit");
      do {
	gotoxy (comx,comy + 5);
	cputs ("Response:  ");
	gotoxy (wherex() - 1,wherey());
	command = getch();
      } while ((command != 'e')&&(command != 'r')&&(command != 'n')
	    &&(command != 'x'));
      return command;
      } /*  function  */

void pattern_feedforward(ff_setup_rec setup,
     weights_rec weights,training_rec tr_rec,pattern the_pattern) {
     const
       netx = 40,  /* Used for screen positioning */
       nety = 8;
     f_vector x = NULL,y = NULL;
     network_rec network = {0,NULL,NULL,NULL};
     int i,j,k;
     short hor_pos;
     allocate_network (setup,&network);
     x = allocate_f_vector (1,setup.no_of_inputs);
     y = allocate_f_vector (1,setup.no_of_outputs);
     for (i = 1;i<=setup.no_of_inputs;i++)
	 x[i] = the_pattern.nodes[i-1];
     normalize_data (x,network.x_inp,setup.no_of_inputs,
	 tr_rec.max_in,tr_rec.min_in,setup.norm_in_range,setup.norm_in_min);
     feedforward (network,weights,setup);
     denormalize_output (network.y_out,y,setup.no_of_outputs,
	setup.norm_out_range,setup.norm_out_min,tr_rec);
     /* Display output results:  */
     gotoxy (netx,nety);
     cputs ("Net Output: ");
     for (i = 1;i<=setup.no_of_outputs;i++) {
	 k = scale (network.y_out[i]);
	 hor_pos = netx + (i -1) * 3;
	 for (j = 1;j<=5;j++) {
	     gotoxy (hor_pos,nety + 7 - j);
	     putch (BLOCK_1);
	     } /* j */
	 for (j = 1;j<=k;j++) {
	     gotoxy (hor_pos,nety + 7 - j);
	     putch (BLOCK_4);
	     } /* j */
	 gotoxy (hor_pos,nety + 8);
	 cprintf ("%1d",i);
	 }  /* i */
     free_f_vector (y,1);
     free_f_vector (x,1);
     free_network (&network);
     return;
     }  /* proc */

void run_pattern_network(ff_setup_rec setup,pattern_display_data *disp_data) {
     char resp;
     path_str pattern_file_name;
     pattern_array *ppatterns = NULL;
     pattern the_pattern;
     int pattern_no = 0;
     weights_rec weights = {0,NULL,NULL};
     training_rec tr_rec;
     clrscr();
     textcolor (YELLOW);
     gotoxy (1,3);
     cputs ("Neural Net Pattern Recognition Program");
     gotoxy (1,5);
     if (setup.files.in_wts_file_name[0] == '\0') {
	cprintf ("\r\nNetwork Weights file name is blank.\r\n");
	cputs ("Press key to cancel: ");
	getch();
	return;
	}
     if (setup.files.training_file_name[0] == '\0') {
	cprintf ("\r\nTraining file name is blank.\r\n");
	cputs ("Press key to cancel: ");
	getch();
	return;
	}
     if (setup.files.data_file_name[0] == '\0') {
	cprintf ("\r\nPattern Data file name is blank.\r\n");
	cputs ("Press key to cancel: ");
	getch();
	return;
	}
     cprintf ("Input weights file name: %s.",
	     setup.files.in_wts_file_name);
     gotoxy (1,7);
     cprintf ("Training file name: %s.",setup.files.training_file_name);
     gotoxy (1,8);
     cprintf ("Input pattern data file: %s.",setup.files.data_file_name);
     gotoxy (1,11);
     cputs ("Press any key to continue (esc to cancel): ");
     if (getch() == ESC_KEY) return;
     clrscr();
     ppatterns = (pattern_array *)malloc((size_t)sizeof(pattern_array));
     if (!process_training_file (setup.files.training_file_name,
	setup.no_of_inputs,setup.no_of_outputs,&tr_rec))
	goto exit_proc;
     if (!read_weights (&weights,setup)) goto exit_proc;
     if (!read_patterns (disp_data,setup.files.data_file_name,
	*ppatterns)) goto exit_proc;

     clrscr();
     if (!get_pattern (disp_data,*ppatterns,&pattern_no)) goto exit_proc;
     the_pattern = (*ppatterns)[pattern_no];

  do {
    resp = get_command ();
    switch (resp) {
    case 'e':
	edit_pattern (disp_data,&the_pattern,pattern_no);
	display_pattern (disp_data,&the_pattern,pattern_no);
	break;
    case 'r':
	pattern_feedforward (setup,weights,tr_rec,the_pattern);
	break;
    case 'n':
	clrscr();
	if (get_pattern (disp_data,*ppatterns,&pattern_no))
	   the_pattern = (*ppatterns)[pattern_no];
	break;
	} /* switch  */
    } while (resp != 'x');

exit_proc:
    textmode (LASTMODE);
    if (ppatterns) free (ppatterns);
    free_training_pointers(&tr_rec);
    free_weights (&weights);
    return;

    } /* proc  */

#endif