// File SLFSETUP.CPP Set up Turbo Vision interface for
//      Self Organizing Graphic Network

#ifndef SLFSETUP_CPP
#define SLFSETUP_CPP

#include <conio.h>
#include <string.h>

#include "uisetup.cpp"

extern "C" {
   #include "slforg.h"
   }

int set_dynam_system;
extern const char dyn_descr [2][DYN_DESCR_LEN] =
      {"Uniform Distribution","Henon System"};

void initialize_self_org_setup (self_org_setup_rec *the_rec)
  {
  the_rec->the_sys = unif_dist;
  strcpy(the_rec->sys_descr,dyn_descr[the_rec->the_sys]);
  the_rec->lattice_rows = 10;
  the_rec->lattice_cols = 10;
  the_rec->a = 1.4;
  the_rec->b = 0.3;
  the_rec->freq = 5;
  the_rec->starting_stepsize = 0.25;
  the_rec->starting_nbhd = 5;
  the_rec->iter_interval = 500;
  the_rec->color = LIGHTGREEN;
  the_rec->gr_rec.scale_window_from_data = 1;
  the_rec->gr_rec.max_mode = VGAMED;
  the_rec->gr_rec.x_min = -1.0;
  the_rec->gr_rec.x_max = 1.0;
  the_rec->gr_rec. y_min = -1.0;
  the_rec->gr_rec.y_max = 1.0;
  the_rec->gr_rec.back_color = BLUE;
  the_rec->gr_rec.text_color = YELLOW;
  the_rec->gr_rec.line_color = LIGHTCYAN;
  }

void slf_setup_to_collection (self_org_setup_rec *the_slf_setup,
   TDataCollection *the_collection)
   {
      void * value_ptr = NULL;
      tdata_type the_type;
      unsigned short len;
      boolean the_check = TRUE;
      unsigned short dec_places = 0;
      float the_max = 0.0;
      float the_min = 0.0;
      boolean enable = TRUE;
      int item_no = 0;

   value_ptr = &the_slf_setup->sys_descr;
   the_type = PATH_DATA;  // get_new_value will be intercepted
   len = DYN_DESCR_LEN;
   item_no++;
   the_collection->insert (new tcolor_data_obj (
	 "Dynam System",
	 value_ptr,the_type,len,the_check,dec_places,the_max,the_min,
	 enable,"",item_no));
   set_dynam_system = item_no;

   value_ptr = &the_slf_setup->a;
   the_type = REAL_DATA;
   len = 6;
   dec_places = 4;
   the_max = 2.1;
   the_min = -0.9;
   item_no++;
   if (the_slf_setup->the_sys != Henon_sys) enable = FALSE;
   the_collection->insert (new tcolor_data_obj (
	 "Parameter A",
	 value_ptr,the_type,len,the_check,dec_places,the_max,the_min,
	 enable,"",item_no));

   value_ptr = &the_slf_setup->b;
   the_type = REAL_DATA;
   the_max = 1.5;
   the_min = -1.4;
   item_no++;
   if (the_slf_setup->the_sys != Henon_sys) enable = FALSE;
   the_collection->insert (new tcolor_data_obj (
	 "Parameter B",
	 value_ptr,the_type,len,the_check,dec_places,the_max,the_min,
	 enable,"",item_no));

   value_ptr = &the_slf_setup->starting_stepsize;
   the_type = REAL_DATA;
   the_max = 1.0;
   the_min = 0.0;
   item_no++;
   enable = TRUE;
   the_collection->insert (new tcolor_data_obj (
	 "Starting Stepsize",
	 value_ptr,the_type,len,the_check,dec_places,the_max,the_min,
	 enable,"",item_no));

   value_ptr = &the_slf_setup->starting_nbhd;
   the_type = INT_DATA;
   len = 3;
   dec_places = 0;
   the_max = 100;
   the_min = 1;
   item_no++;
   the_collection->insert (new tcolor_data_obj (
	 "Starting Nbhd Size",
	 value_ptr,the_type,len,the_check,dec_places,the_max,the_min,
	 enable,"",item_no));

   value_ptr = &the_slf_setup->freq;
   the_type = INT_DATA;
   len = 3;
   dec_places = 0;
   the_max = 100;
   the_min = 1;
   item_no++;
   the_collection->insert (new tcolor_data_obj (
	 "Graph Frequency",
	 value_ptr,the_type,len,the_check,dec_places,the_max,the_min,
	 enable,"",item_no));

   value_ptr = &the_slf_setup->iter_interval;
   the_type = INT_DATA;
   len = 5;
   dec_places = 0;
   the_max = 10000;
   the_min = 1;
   item_no++;
   the_collection->insert (new tcolor_data_obj (
	 "Iteration Interval",
	 value_ptr,the_type,len,the_check,dec_places,the_max,the_min,
	 enable,"",item_no));

   value_ptr = &the_slf_setup->lattice_rows;
   the_type = INT_DATA;
   the_max = 100;
   the_min = 1;
   item_no++;
   the_collection->insert (new tcolor_data_obj (
	 "Lattice Rows",
	 value_ptr,the_type,len,the_check,dec_places,the_max,the_min,
	 enable,"",item_no));

   value_ptr = &the_slf_setup->lattice_cols;
   the_type = INT_DATA;
   the_max = 100;
   the_min = 1;
   item_no++;
   the_collection->insert (new tcolor_data_obj (
	 "Lattice Cols",
	 value_ptr,the_type,len,the_check,dec_places,the_max,the_min,
	 enable,"",item_no));

   value_ptr = &the_slf_setup->color;
   the_type = VGA_COLOR_DATA;
   len = 2;
   dec_places = 0;
   the_max = 15;
   the_min = 0;
   item_no++;
   the_collection->insert (new tcolor_data_obj (
	 "Net Color",
	 value_ptr,the_type,len,the_check,dec_places,the_max,the_min,
	 enable,"",item_no));

   value_ptr = &the_slf_setup->gr_rec.scale_window_from_data;
   the_type = BOOLEAN_DATA;
   item_no++;
   the_collection->insert (new tcolor_data_obj (
	 "Scale Window from Data?",
	 value_ptr,the_type,len,the_check,dec_places,the_max,the_min,
	 enable,"",item_no));

   value_ptr = &the_slf_setup->gr_rec.x_min;
   the_type = REAL_DATA;
   the_check = FALSE;
   len = 6;
   dec_places = 4;
   item_no++;
   the_collection->insert (new tcolor_data_obj (
	 "Window X Min",
	 value_ptr,the_type,len,the_check,dec_places,the_max,the_min,
	 enable,"",item_no));

   value_ptr = &the_slf_setup->gr_rec.x_max;
   item_no++;
   the_collection->insert (new tcolor_data_obj (
	 "Window X Max",
	 value_ptr,the_type,len,the_check,dec_places,the_max,the_min,
	 enable,"",item_no));

   value_ptr = &the_slf_setup->gr_rec.y_min;
   item_no++;
   the_collection->insert (new tcolor_data_obj (
	 "Window Y Min",
	 value_ptr,the_type,len,the_check,dec_places,the_max,the_min,
	 enable,"",item_no));

   value_ptr = &the_slf_setup->gr_rec.y_max;
   item_no++;
   the_collection->insert (new tcolor_data_obj (
	 "Window Y Max",
	 value_ptr,the_type,len,the_check,dec_places,the_max,the_min,
	 enable,"",item_no));

   the_type = VGA_COLOR_DATA;
   value_ptr = &the_slf_setup->gr_rec.back_color;
   the_check = TRUE;
   len = 2;
   dec_places = 0;
   the_max = 15;
   the_min = 0;
   item_no++;
   the_collection->insert (new tcolor_data_obj (
	 "Background Color",
	 value_ptr,the_type,len,the_check,dec_places,the_max,the_min,
	 enable,"",item_no));

   value_ptr = &the_slf_setup->gr_rec.line_color;
   item_no++;
   the_collection->insert (new tcolor_data_obj (
	 "Line Color",
	 value_ptr,the_type,len,the_check,dec_places,the_max,the_min,
	 enable,"",item_no));

   value_ptr = &the_slf_setup->gr_rec.text_color;
   item_no++;
   the_collection->insert (new tcolor_data_obj (
	 "Text Color",
	 value_ptr,the_type,len,the_check,dec_places,the_max,the_min,
	 enable,"",item_no));
   return;
   }

class tslfsetapp : public TSetApp
   {
   public :
     tslfsetapp();
     virtual void setup();
     static TMenuBar *initMenuBar (TRect r );
     static void init_submenus();
     virtual char *save_setup (char *the_file_name);
     virtual char *read_setup (char *the_file_name);
   };

tslfsetapp::tslfsetapp() : TSetApp(),
   TProgInit (&tslfsetapp::initStatusLine,
	      &tslfsetapp::initMenuBar,
	      &tslfsetapp::initDeskTop
	      )
   {
   slf_setup_to_collection (&slf_rec,system_setup_data.item_collection);
   if (quit_flag == startup) about_dialog();
   }

 void tslfsetapp::init_submenus()
   {
   TSetApp::init_submenus();
   // Skip train, data and special submenus:
   setup_submenu->next = NULL;
   file_submenu->subMenu->items = quit_item;
   file_submenu->subMenu->deflt = quit_item;
   }

TMenuBar *tslfsetapp::initMenuBar ( TRect r )
{
   r.b.y = r.a.y + 1;  //Set bottom line 1 line below top line
   init_submenus();
   return new TMenuBar (r,new TMenu(*about_submenu));
 }

void dyn_sys_processing(tlist_box_data *the_data_rec,int item_no)
   {
   const two_items = 2;
   char the_title [16] = "Dyn System Type";
   char the_descr [22] = "Dynamical System Type";
   ushort i;
   ushort the_type;

   if (item_no == set_dynam_system) {
      the_type = slf_rec.the_sys + 1;
      radio_dialog (the_title,the_descr,(void *)dyn_descr,two_items,
	  DYN_DESCR_LEN,&the_type);
      slf_rec.the_sys = (dynam_sys) (the_type - 1);
      strcpy(slf_rec.sys_descr,dyn_descr[the_type -1]);
      switch (slf_rec.the_sys) {
      case unif_dist:
      // Disable Henon parameter items
	 for (i = 2;i <= 3;i++)
       ((tcolor_data_obj *) (the_data_rec->item_collection)->at(i - 1))
	 ->disable();
	 break;
      case Henon_sys:
      // Enable parameter items
	 for (i = 2;i <= 3;i++)
       ((tcolor_data_obj *) (the_data_rec->item_collection)->at(i - 1))
	 ->enable();
	 break;
      }  // end switch
      return;
      }  // end if
   ((tcolor_data_obj *) (the_data_rec->item_collection)->at(item_no - 1))
     ->get_new_value();
   return;
   }

void tslfsetapp::setup()
   {
   list_dialog ("SETUP PARAMETERS","",&system_setup_data,
     MAX_NO_OF_SETUP_ITEMS,&dyn_sys_processing);
   return;
   }

char *tslfsetapp::save_setup (char *the_file_name)
   {
   write_record (the_file_name,(void *)&slf_rec,sizeof(slf_rec));
   return the_file_name;
   }

char *tslfsetapp::read_setup (char *the_file_name)
   {
   read_record (the_file_name,(void *)&slf_rec,sizeof(slf_rec));
   return the_file_name;
   }

#endif